// *** THIS FILE IS GENERATED - DO NOT EDIT ***
// See shader_object_generator.py for modifications

/*
 * Copyright 2023-2025 Nintendo
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

VkPhysicalDeviceDynamicRenderingFeatures dynamic_rendering;
VkPhysicalDevicePrivateDataFeaturesEXT private_data;
VkPhysicalDeviceExtendedDynamicStateFeaturesEXT extended_dynamic_state_1;
VkPhysicalDeviceExtendedDynamicState2FeaturesEXT extended_dynamic_state_2;
VkPhysicalDeviceExtendedDynamicState3FeaturesEXT extended_dynamic_state_3;
VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT vertex_input_dynamic;
VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT graphics_pipeline_library;
VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT dynamic_rendering_unused_attachments;
VkPhysicalDeviceTransformFeedbackFeaturesEXT transform_feedback;
VkPhysicalDeviceDepthClipEnableFeaturesEXT depth_clip_enable;
VkPhysicalDeviceProvokingVertexFeaturesEXT provoking_vertex;
VkPhysicalDeviceLineRasterizationFeaturesEXT line_rasterization;
VkPhysicalDeviceDepthClipControlFeaturesEXT depth_clip_control;
VkPhysicalDeviceCoverageReductionModeFeaturesNV nv_coverage_reduction_mode;
VkPhysicalDeviceShadingRateImageFeaturesNV nv_shading_rate_image;
VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV nv_representative_fragment_test;
