
;;;; Begin Copyright Notice ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2023      Nvidia Corporation
; SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
; SPDX-License-Identifier: Apache-2.0 or MIT
;
; Licensed under either of
;   Apache License, Version 2.0 (http://www.apache.org/licenses/LICENSE-2.0)
;   or
;   MIT license (http://opensource.org/licenses/MIT)
; at your option.
;
; Any contribution submitted by you to Khronos for inclusion in this work shall be dual licensed as above.
;
; Author: Vikram Kushwaha <vkushwaha@nvidia.com>
;
;;;;  End Copyright Notice ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LIBRARY VkLayer_khronos_memory_decompression
EXPORTS
vkGetInstanceProcAddr
vkGetDeviceProcAddr
vkEnumerateInstanceExtensionProperties
vkNegotiateLoaderLayerInterfaceVersion
