// *** THIS FILE IS GENERATED - DO NOT EDIT ***
// See shader_object_generator.py for modifications

/*
 * Copyright 2023-2025 Nintendo
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

device_data->dynamic_rendering = dynamic_rendering_ptr ? *dynamic_rendering_ptr : dynamic_rendering_local;
device_data->private_data = private_data_ptr ? *private_data_ptr : private_data_local;
device_data->extended_dynamic_state_1 = extended_dynamic_state_1_ptr ? *extended_dynamic_state_1_ptr : extended_dynamic_state_1_local;
device_data->extended_dynamic_state_2 = extended_dynamic_state_2_ptr ? *extended_dynamic_state_2_ptr : extended_dynamic_state_2_local;
device_data->extended_dynamic_state_3 = extended_dynamic_state_3_ptr ? *extended_dynamic_state_3_ptr : extended_dynamic_state_3_local;
device_data->vertex_input_dynamic = vertex_input_dynamic_ptr ? *vertex_input_dynamic_ptr : vertex_input_dynamic_local;
device_data->graphics_pipeline_library = graphics_pipeline_library_ptr ? *graphics_pipeline_library_ptr : graphics_pipeline_library_local;
device_data->dynamic_rendering_unused_attachments = dynamic_rendering_unused_attachments_ptr ? *dynamic_rendering_unused_attachments_ptr : dynamic_rendering_unused_attachments_local;
device_data->transform_feedback = transform_feedback_ptr ? *transform_feedback_ptr : transform_feedback_local;
device_data->depth_clip_enable = depth_clip_enable_ptr ? *depth_clip_enable_ptr : depth_clip_enable_local;
device_data->provoking_vertex = provoking_vertex_ptr ? *provoking_vertex_ptr : provoking_vertex_local;
device_data->line_rasterization = line_rasterization_ptr ? *line_rasterization_ptr : line_rasterization_local;
device_data->depth_clip_control = depth_clip_control_ptr ? *depth_clip_control_ptr : depth_clip_control_local;
device_data->nv_coverage_reduction_mode = nv_coverage_reduction_mode_ptr ? *nv_coverage_reduction_mode_ptr : nv_coverage_reduction_mode_local;
device_data->nv_shading_rate_image = nv_shading_rate_image_ptr ? *nv_shading_rate_image_ptr : nv_shading_rate_image_local;
device_data->nv_representative_fragment_test = nv_representative_fragment_test_ptr ? *nv_representative_fragment_test_ptr : nv_representative_fragment_test_local;
